@@ -1,6 +1,6 @@
 /*
  ** dataModel_SleuthkitJNI
- ** The Sleuth Kit 
+ ** The Sleuth Kit
  **
  ** Brian Carrier [carrier <at> sleuthkit [dot] org]
  ** Copyright (c) 2010-2018 Brian Carrier.  All Rights reserved
@@ -43,9 +43,9 @@ static std::vector<TSK_HDB_INFO *> hashDbs;
 * TSK_FS_FILE still needs be maintained for opening and closing.
 */
 typedef struct {
-    uint32_t tag; 
-    TSK_FS_FILE *fs_file; 
-    TSK_FS_ATTR *fs_attr; 
+    uint32_t tag;
+    TSK_FS_FILE *fs_file;
+    TSK_FS_ATTR *fs_attr;
 } TSK_JNI_FILEHANDLE;
 #define TSK_JNI_FILEHANDLE_TAG 0x10101214
 
@@ -55,7 +55,7 @@ typedef struct {
 /**
 * Sets flag to throw an TskCoreException back up to the Java code with a specific message.
 * Note: exception is thrown to Java code after the native function returns
-* not when setThrowTskCoreError() is invoked - this must be addressed in the code following the exception 
+* not when setThrowTskCoreError() is invoked - this must be addressed in the code following the exception
 * @param the java environment to send the exception to
 * @param msg message string
  */
@@ -70,7 +70,7 @@ setThrowTskCoreError(JNIEnv * env, const char *msg)
 /**
 * Sets flag to throw an TskCoreException back up to the Java code with the currently set error message.
 * Note: exception is thrown to Java code after the native function returns
-* not when setThrowTskCoreError() is invoked - this must be addressed in the code following the exception 
+* not when setThrowTskCoreError() is invoked - this must be addressed in the code following the exception
 * @param the java environment to send the exception to
 */
 static void
@@ -83,7 +83,7 @@ setThrowTskCoreError(JNIEnv * env)
 /**
 * Sets flag to throw an TskDataException back up to the Java code with a specific message.
 * Note: exception is thrown to Java code after the native function returns
-* not when setThrowTskDataError() is invoked - this must be addressed in the code following the exception 
+* not when setThrowTskDataError() is invoked - this must be addressed in the code following the exception
 * @param the java environment to send the exception to
 * @param msg message string
  */
@@ -99,7 +99,7 @@ setThrowTskDataError(JNIEnv * env, const char *msg)
 /**
 * Sets flag to throw an TskDataException back up to the Java code with the currently set error message.
 * Note: exception is thrown to Java code after the native function returns
-* not when setThrowTskDataError() is invoked - this must be addressed in the code following the exception 
+* not when setThrowTskDataError() is invoked - this must be addressed in the code following the exception
 * @param the java environment to send the exception to
 */
 static void
@@ -212,7 +212,7 @@ castJniFileHandle(JNIEnv * env, jlong ptr)
     return lcl;
 }
 
-static TskCaseDb * 
+static TskCaseDb *
 castCaseDb(JNIEnv * env, jlong ptr)
 {
     TskCaseDb *lcl = ((TskCaseDb *) ptr);
@@ -230,7 +230,7 @@ castCaseDb(JNIEnv * env, jlong ptr)
  * @param buffer Buffer to store resulting string into
  * @param size Length of buffer
  * @param strJ string to convert
- * @returns 1 on error 
+ * @returns 1 on error
  */
 static int
 toTCHAR(JNIEnv * env, TSK_TCHAR * buffer, size_t size, jstring strJ)
@@ -278,7 +278,7 @@ toTCHAR(JNIEnv * env, TSK_TCHAR * buffer, size_t size, jstring strJ)
  */
 JNIEXPORT jint JNICALL
     Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbOpenNat(JNIEnv * env,
-    jclass obj, jstring pathJ) 
+    jclass obj, jstring pathJ)
 {
     TSK_TCHAR pathT[1024];
     toTCHAR(env, pathT, 1024, pathJ);
@@ -288,7 +288,7 @@ JNIEXPORT jint JNICALL
         setThrowTskCoreError(env, tsk_error_get_errstr());
         return -1;
     }
-    
+
     // The index of the pointer in the vector is used as a handle for the
     // database.
     hashDbs.push_back(db);
@@ -321,7 +321,7 @@ JNIEXPORT jint JNICALL
 
     // The index of the pointer in the vector is used as a handle for the
     // database.
-    hashDbs.push_back(db);    
+    hashDbs.push_back(db);
     return (jint)hashDbs.size();
 }
 
@@ -438,7 +438,7 @@ JNIEXPORT jint JNICALL
     const char * sha1 = hashSha1J ? (const char *) env->GetStringUTFChars(hashSha1J, &isCopy) : NULL;
     const char * sha256 = hashSha256J ? (const char *) env->GetStringUTFChars(hashSha256J, &isCopy) : NULL;
     const char * comment = commentJ ? (const char *) env->GetStringUTFChars(commentJ, &isCopy) : NULL;
-   
+
     if (tsk_hdb_add_entry(db, name, md5, sha1, sha256, comment)) {
         setThrowTskCoreError(env, tsk_error_get_errstr());
     }
@@ -447,7 +447,7 @@ JNIEXPORT jint JNICALL
         env->ReleaseStringUTFChars(filenameJ, (const char *) name);
     }
 
-    if (hashMd5J) { 
+    if (hashMd5J) {
         env->ReleaseStringUTFChars(hashMd5J, (const char *) md5);
     }
 
@@ -514,10 +514,10 @@ JNIEXPORT jboolean JNICALL
         return (jboolean)false;
     }
 
-    return (jboolean)((tsk_hdb_uses_external_indexes(db) == 1) && 
+    return (jboolean)((tsk_hdb_uses_external_indexes(db) == 1) &&
                       (tsk_hdb_is_idx_only(db) == 0));
 }
- 
+
 /**
  * Gets the path of a hash database.
  * @param env Pointer to Java environment from which this method was called.
@@ -600,7 +600,7 @@ JNIEXPORT jstring JNICALL
  * @param env Pointer to Java environment from which this method was called.
  * @param obj The Java object from which this method was called.
  * @param dbHandle A handle for the hash database.
- * @return True if the hash database is an external index serving as a 
+ * @return True if the hash database is an external index serving as a
  * database.
  */
 JNIEXPORT jboolean JNICALL
@@ -658,7 +658,7 @@ JNIEXPORT jstring JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbGetDis
  */
 JNIEXPORT void JNICALL
     Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbCloseAll(JNIEnv * env,
-    jclass obj) 
+    jclass obj)
 {
     for (std::vector<TSK_HDB_INFO *>::iterator it = hashDbs.begin(); it != hashDbs.end(); ++it) {
         if (NULL != *it) {
@@ -677,7 +677,7 @@ JNIEXPORT void JNICALL
  */
 JNIEXPORT void JNICALL
     Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbClose(JNIEnv * env,
-    jclass obj, jint dbHandle) 
+    jclass obj, jint dbHandle)
 {
     if((size_t)dbHandle > hashDbs.size()) {
         setThrowTskCoreError(env, "Invalid database handle");
@@ -705,7 +705,7 @@ JNIEXPORT void JNICALL
  * @return True if the hash is found in the hash database, false otherwise.
  */
 JNIEXPORT jboolean JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbLookup
-(JNIEnv * env, jclass obj, jstring hash, jint dbHandle) 
+(JNIEnv * env, jclass obj, jstring hash, jint dbHandle)
 {
     if ((size_t)dbHandle > hashDbs.size()) {
         setThrowTskCoreError(env, "Invalid database handle");
@@ -724,7 +724,7 @@ JNIEXPORT jboolean JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbLooku
     int8_t retval = tsk_hdb_lookup_str(db, cHashStr, TSK_HDB_FLAG_QUICK, NULL, NULL);
     if (retval == -1) {
         setThrowTskCoreError(env, tsk_error_get_errstr());
-    } 
+    }
     else if (retval) {
         file_known = true;
     }
@@ -753,13 +753,13 @@ JNIEXPORT jobject JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbLookup
         setThrowTskCoreError(env, "Invalid database handle");
         return NULL;
     }
-    
+
     jboolean isCopy;
     const char *inputHash = (const char *) env->GetStringUTFChars(hash, &isCopy);
-    TskHashInfo result; 
+    TskHashInfo result;
     int8_t returnCode = tsk_hdb_lookup_verbose_str(db, inputHash, (void*)&result);
     env->ReleaseStringUTFChars(hash, (const char *) inputHash);
-    
+
     if (returnCode == -1) {
         setThrowTskCoreError(env, tsk_error_get_errstr());
         return NULL;
@@ -772,10 +772,10 @@ JNIEXPORT jobject JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbLookup
     // the Java version of a HashInfo object.
     const char *md5 = result.hashMd5.c_str();
     jstring md5j = env->NewStringUTF(md5);
-            
+
     const char *sha1 = result.hashSha1.c_str();
     jstring sha1j = env->NewStringUTF(sha1);
-            
+
     const char *sha256 = result.hashSha2_256.c_str();
     jstring sha256j = env->NewStringUTF(sha256);
 
@@ -797,6 +797,25 @@ JNIEXPORT jobject JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbLookup
     return hashInfo;
 }
 
+/*
+ * Initialize a process for adding an image to a case database.
+ *
+ * @param env Pointer to java environment.
+ * @param obj Pointer the Java class object.
+ * @param timeZone The time zone for the image.
+ * @param addUnallocSpace Pass true to create virtual files for unallocated space. Ignored if addFileSystems is false.
+ * @param skipFatFsOrphans Pass true to skip processing of orphan files for FAT file systems. Ignored if addFileSystems is false.
+ * @param hostId Id of the host (already in the database).
+ * @param passwordJ Password for the file system or null for no password.
+ *
+ * @return A pointer to the process (TskAutoDbJava object) or NULL on error.
+ */
+JNIEXPORT jlong JNICALL
+    Java_org_sleuthkit_datamodel_SleuthkitJNI_initAddImgNatPassword(JNIEnv * env,
+    jclass obj, jobject callbackObj, jstring timeZone, jboolean addUnallocSpace, jboolean skipFatFsOrphans, jstring passwordJ) {
+    return Java_org_sleuthkit_datamodel_SleuthkitJNI_initializeAddImgPasswordNat(env, obj, callbackObj, timeZone, true, addUnallocSpace, skipFatFsOrphans, passwordJ);
+}
+
 /*
  * Initialize a process for adding an image to a case database.
  *
@@ -812,7 +831,7 @@ JNIEXPORT jobject JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbLookup
 JNIEXPORT jlong JNICALL
     Java_org_sleuthkit_datamodel_SleuthkitJNI_initAddImgNat(JNIEnv * env,
     jclass obj, jobject callbackObj, jstring timeZone, jboolean addUnallocSpace, jboolean skipFatFsOrphans) {
-    return Java_org_sleuthkit_datamodel_SleuthkitJNI_initializeAddImgNat(env, obj, callbackObj, timeZone, true, addUnallocSpace, skipFatFsOrphans);
+    return Java_org_sleuthkit_datamodel_SleuthkitJNI_initializeAddImgPasswordNat(env, obj, callbackObj, timeZone, true, addUnallocSpace, skipFatFsOrphans, NULL);
 }
 
 /*
@@ -824,13 +843,31 @@ JNIEXPORT jlong JNICALL
  * @param addFileSystems Pass true to attempt to add file systems within the image to the case database.
  * @param addUnallocSpace Pass true to create virtual files for unallocated space. Ignored if addFileSystems is false.
  * @param skipFatFsOrphans Pass true to skip processing of orphan files for FAT file systems. Ignored if addFileSystems is false.
- * @param hostId The ID of the host (already in database).
  *
  * @return A pointer to the process (TskAutoDbJava object) or NULL on error.
  */
 JNIEXPORT jlong JNICALL
 Java_org_sleuthkit_datamodel_SleuthkitJNI_initializeAddImgNat(JNIEnv * env, jclass obj,
     jobject callbackObj, jstring timeZone, jboolean addFileSystems, jboolean addUnallocSpace, jboolean skipFatFsOrphans) {
+    return Java_org_sleuthkit_datamodel_SleuthkitJNI_initializeAddImgPasswordNat(env, obj, callbackObj, timeZone, true, addUnallocSpace, skipFatFsOrphans, NULL);
+}
+
+/*
+ * Initialize a process for adding an image to a case database.
+ *
+ * @param env Pointer to java environment.
+ * @param obj Pointer the Java class object.
+ * @param timeZone The time zone for the image.
+ * @param addFileSystems Pass true to attempt to add file systems within the image to the case database.
+ * @param addUnallocSpace Pass true to create virtual files for unallocated space. Ignored if addFileSystems is false.
+ * @param skipFatFsOrphans Pass true to skip processing of orphan files for FAT file systems. Ignored if addFileSystems is false.
+ * @param passwordJ Password for the file system or null for no password.
+ *
+ * @return A pointer to the process (TskAutoDbJava object) or NULL on error.
+ */
+JNIEXPORT jlong JNICALL
+Java_org_sleuthkit_datamodel_SleuthkitJNI_initializeAddImgPasswordNat(JNIEnv * env, jclass obj,
+    jobject callbackObj, jstring timeZone, jboolean addFileSystems, jboolean addUnallocSpace, jboolean skipFatFsOrphans, jstring passwordJ) {
     jboolean isCopy;
 
     if (env->GetStringUTFLength(timeZone) > 0) {
@@ -866,6 +903,13 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_initializeAddImgNat(JNIEnv * env, jcla
         return 0;
     }
 
+    if (passwordJ != NULL) {
+        jboolean isCopyPassword;
+        const char* password = (const char*)env->GetStringUTFChars(passwordJ, &isCopyPassword);
+        tskAutoJava->setFileSystemPassword(string(password));
+        env->ReleaseStringUTFChars(passwordJ, password);
+    }
+
     // set the options flags
     tskAutoJava->setAddFileSystems(addFileSystems?true:false);
     if (addFileSystems) {
@@ -893,7 +937,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_initializeAddImgNat(JNIEnv * env, jcla
 
 /*
  * Add an image to a database using a pre-created process, which can be cancelled.
- * MUST call commitAddImg or revertAddImg afterwards once runAddImg returns.  If there is an 
+ * MUST call commitAddImg or revertAddImg afterwards once runAddImg returns.  If there is an
  * error, you do not need to call revert or commit and the 'process' handle will be deleted.
  *
  * @param env pointer to java environment this was called from
@@ -910,14 +954,14 @@ JNIEXPORT void JNICALL
 
     TskAutoDbJava *tskAuto = ((TskAutoDbJava *) process);
     if (!tskAuto || tskAuto->m_tag != TSK_AUTO_TAG) {
-        setThrowTskCoreError(env, 
+        setThrowTskCoreError(env,
             "runOpenAndAddImgNat: Invalid TskAutoDbJava object passed in");
         return;
     }
 
     jboolean isCopy;
     const char *device_id = NULL;
-    if (NULL != deviceId) {    
+    if (NULL != deviceId) {
         device_id = (const char *) env->GetStringUTFChars(deviceId, &isCopy);
         if (NULL == device_id) {
             setThrowTskCoreError(env, "runOpenAndAddImgNat: Can't convert data source id string");
@@ -1009,7 +1053,7 @@ JNIEXPORT void JNICALL
 JNIEXPORT void JNICALL
 Java_org_sleuthkit_datamodel_SleuthkitJNI_runAddImgNat(JNIEnv * env,
     jclass obj, jlong process, jstring deviceId, jlong a_img_info, jlong img_id, jstring timeZone, jstring imageWriterPathJ) {
-    
+
     TskAutoDbJava *tskAuto = ((TskAutoDbJava *)process);
     if (!tskAuto || tskAuto->m_tag != TSK_AUTO_TAG) {
         setThrowTskCoreError(env,
@@ -1224,7 +1268,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_getPathsForImageNat(JNIEnv * env,
         }
         img_ptrs[i] = img2;
     }
-#else 
+#else
     img_ptrs = img_info->images;
 #endif
 
@@ -1300,7 +1344,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_getMD5HashForImageNat(JNIEnv * env,
         return 0;
     }
     // env->NewStringUTF(img_ptrs[i])
-#if HAVE_LIBEWF 
+#if HAVE_LIBEWF
     if (img_info->itype == TSK_IMG_TYPE_EWF_EWF) {
         IMG_EWF_INFO *ewf_info = (IMG_EWF_INFO *)img_info;
         if (ewf_info->md5hash_isset) {
@@ -1324,7 +1368,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_getSha1HashForImageNat(JNIEnv * env,
         return 0;
     }
     // env->NewStringUTF(img_ptrs[i])
-#if HAVE_LIBEWF 
+#if HAVE_LIBEWF
     if (img_info->itype == TSK_IMG_TYPE_EWF_EWF) {
         IMG_EWF_INFO *ewf_info = (IMG_EWF_INFO *)img_info;
         if (ewf_info->sha1hash_isset) {
@@ -1348,7 +1392,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_getCollectionDetailsForImageNat(JNIEnv
         return 0;
     }
     // env->NewStringUTF(img_ptrs[i])
-#if HAVE_LIBEWF 
+#if HAVE_LIBEWF
     if (img_info->itype == TSK_IMG_TYPE_EWF_EWF) {
         IMG_EWF_INFO *ewf_info = (IMG_EWF_INFO *)img_info;
         ewf_get_details(ewf_info);
@@ -1463,19 +1507,55 @@ JNIEXPORT jlong JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_getImgInfoForP
  * @param env pointer to java environment this was called from
  * @param obj the java object this was called from
  * @param a_img_info the pointer to the parent img object
- * @param fs_offset the offset in bytes to the file system 
+ * @param fs_offset the offset in bytes to the file system
  */
 JNIEXPORT jlong JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_openFsNat
     (JNIEnv * env, jclass obj, jlong a_img_info, jlong fs_offset) {
+    TSK_IMG_INFO* img_info = castImgInfo(env, a_img_info);
+    if (img_info == 0) {
+        //exception already set
+        return 0;
+    }
+
+    TSK_FS_INFO* fs_info;
+    fs_info =
+        tsk_fs_open_img(img_info, (TSK_OFF_T)fs_offset,
+            TSK_FS_TYPE_DETECT);
+    if (fs_info == NULL) {
+        setThrowTskCoreError(env, tsk_error_get());
+    }
+    return (jlong)fs_info;
+}
+
+/*
+ * Open file system with the given offset
+ * @return the created TSK_FS_INFO pointer
+ * @param env pointer to java environment this was called from
+ * @param obj the java object this was called from
+ * @param a_img_info the pointer to the parent img object
+ * @param fs_offset the offset in bytes to the file system
+ * @param password Password for the file system
+ */
+JNIEXPORT jlong JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_openFsDecryptNat
+    (JNIEnv * env, jclass obj, jlong a_img_info, jlong fs_offset, jstring passwordJ) {
     TSK_IMG_INFO *img_info = castImgInfo(env, a_img_info);
     if (img_info == 0) {
         //exception already set
         return 0;
     }
+
+    const char* password = NULL;
+    jboolean isCopy;
+    if (passwordJ != NULL) {
+        password = (const char*)env->GetStringUTFChars(passwordJ, &isCopy);
+    }
+
     TSK_FS_INFO *fs_info;
     fs_info =
-        tsk_fs_open_img(img_info, (TSK_OFF_T) fs_offset,
-        TSK_FS_TYPE_DETECT);
+        tsk_fs_open_img_decrypt(img_info, (TSK_OFF_T) fs_offset,
+        TSK_FS_TYPE_DETECT, password);
+    env->ReleaseStringUTFChars(passwordJ, (const char*)password);
+
     if (fs_info == NULL) {
         setThrowTskCoreError(env, tsk_error_get());
     }
@@ -1503,7 +1583,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_openFileNat(JNIEnv * env,
         return 0;
     }
 
-    
+
     TSK_FS_FILE *file_info;
     //open file
     file_info = tsk_fs_file_open_meta(fs_info, NULL, (TSK_INUM_T) file_id);
@@ -1513,7 +1593,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_openFileNat(JNIEnv * env,
     }
 
     //open attribute
-    const TSK_FS_ATTR * tsk_fs_attr = 
+    const TSK_FS_ATTR * tsk_fs_attr =
         tsk_fs_file_attr_get_type(file_info, (TSK_FS_ATTR_TYPE_ENUM)attr_type, (uint16_t)attr_id, 1);
     if (tsk_fs_attr == NULL) {
         tsk_fs_file_close(file_info);
@@ -1522,7 +1602,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_openFileNat(JNIEnv * env,
     }
 
     //allocate file handle structure to encapsulate file and attribute
-    TSK_JNI_FILEHANDLE * fileHandle = 
+    TSK_JNI_FILEHANDLE * fileHandle =
         (TSK_JNI_FILEHANDLE *) tsk_malloc(sizeof(TSK_JNI_FILEHANDLE));
     if (fileHandle == NULL) {
         tsk_fs_file_close(file_info);
@@ -1896,7 +1976,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readFsNat(JNIEnv * env,
 
 /**
  * Flag used by readFileNat to specify if the offset is relative to the start of the file
- * or the start of the slack space 
+ * or the start of the slack space
  */
 typedef enum {
     TSK_FS_FILE_READ_OFFSET_TYPE_START_OF_FILE = 0x00,
@@ -1991,13 +2071,13 @@ JNIEXPORT jint JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_saveFileMetaDat
         //exception already set
         return -1;
     }
-    
+
     // check the pointers
     if (file_handle->fs_file == NULL || file_handle->fs_file->fs_info == NULL || file_handle->fs_file->meta == NULL) {
         setThrowTskCoreError(env, "NULL pointers for istat file.");
         return -1;
     }
-    TSK_FS_INFO *fs_info = file_handle->fs_file->fs_info;   
+    TSK_FS_INFO *fs_info = file_handle->fs_file->fs_info;
 
     // open a file to write the details to
     jboolean isCopy;
@@ -2009,7 +2089,7 @@ JNIEXPORT jint JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_saveFileMetaDat
         return -1;
     }
     env->ReleaseStringUTFChars(a_tmp_path, str8);
-    
+
     if (fs_info->istat(fs_info, TSK_FS_ISTAT_RUNLIST, hFile, file_handle->fs_file->meta->addr, 0, 0) != 0) {
         fclose(hFile);
         setThrowTskCoreError(env);
@@ -2101,7 +2181,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_closeFileNat(JNIEnv * env,
         //exception already set
         return;
     }
-    
+
     TSK_FS_FILE * file_info = file_handle->fs_file;
     tsk_fs_file_close(file_info); //also closes the attribute
 
@@ -2198,7 +2278,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbCreateIndexNat (JNIEnv * env,
         // an NSRL hash database.
         TSNPRINTF(idx_type, 1024, _TSK_T("%") PRIcTSK, TSK_HDB_DBTYPE_NSRL_MD5_STR);
     }
-  
+
     if (tsk_hdb_make_index(db, idx_type) != 0) {
         setThrowTskCoreError(env, tsk_error_get_errstr());
     }
@@ -2237,7 +2317,7 @@ JNIEXPORT jboolean JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbIndex
  */
 JNIEXPORT jlong JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_findDeviceSizeNat
   (JNIEnv * env, jclass obj, jstring devPathJ) {
-     
+
       jlong devSize = 0;
       const char* devPath = env->GetStringUTFChars(devPathJ, 0);
 
@@ -2246,7 +2326,7 @@ JNIEXPORT jlong JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_findDeviceSize
         tsk_img_open_utf8_sing(devPath, TSK_IMG_TYPE_DETECT, 0);
       if (img_info == NULL) {
         setThrowTskCoreError(env, tsk_error_get());
-        env->ReleaseStringUTFChars(devPathJ , devPath); 
+        env->ReleaseStringUTFChars(devPathJ , devPath);
         return -1;
       }
 
@@ -2255,7 +2335,7 @@ JNIEXPORT jlong JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_findDeviceSize
 
       //cleanup
       tsk_img_close(img_info);
-      env->ReleaseStringUTFChars(devPathJ , devPath); 
+      env->ReleaseStringUTFChars(devPathJ , devPath);
 
       return devSize;
 }
@@ -2268,36 +2348,81 @@ JNIEXPORT jlong JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_findDeviceSize
  * @return true if the image can be processed, false otherwise
  */
 JNIEXPORT jboolean JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_isImageSupportedNat
-  (JNIEnv * env, jclass obj, jstring imagePathJ) {
-      
+(JNIEnv* env, jclass obj, jstring imagePathJ) {
+
     TskIsImageSupported tskIsImage;
     TSK_TCHAR imagePathT[1024];
     toTCHAR(env, imagePathT, 1024, imagePathJ);
 
     // It seems like passing &imagePathT should work instead of making this new array,
     // but it generated an EXCEPTION_ACCESS_VIOLATION during testing.
-    TSK_TCHAR ** imagePaths = (TSK_TCHAR**)tsk_malloc((1) * sizeof(TSK_TCHAR*));
+    TSK_TCHAR** imagePaths = (TSK_TCHAR**)tsk_malloc((1) * sizeof(TSK_TCHAR*));
     bool result;
     imagePaths[0] = imagePathT;
     if (tskIsImage.openImage(1, imagePaths, TSK_IMG_TYPE_DETECT, 0)) {
         result = false;
-    } else {
+    }
+    else {
         if (tskIsImage.findFilesInImg()) {
             result = false;
-        } else {
+        }
+        else {
             if (tskIsImage.isImageSupported()) {
                 result = true;
             }
             else {
                 result = false;
-            }   
+            }
         }
     }
 
     // Cleanup
     free(imagePaths);
 
-    return (jboolean) result;
+    return (jboolean)result;
+}
+
+/*
+ * Test whether an image is supported
+ * @param env pointer to java environment this was called from
+ * @param obj the java object this was called from
+ * @param imagePathJ the image path
+ * @param passwordJ  the password to try for any file systems found
+ * @return empty string if the image can be processed, error message otherwise
+ */
+JNIEXPORT jstring JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_isImageSupportedStringNat
+  (JNIEnv * env, jclass obj, jstring imagePathJ, jstring passwordJ) {
+
+    TskIsImageSupported tskIsImage;
+    TSK_TCHAR imagePathT[1024];
+    toTCHAR(env, imagePathT, 1024, imagePathJ);
+
+    jboolean isCopy;
+    const char* password = NULL;
+    if (passwordJ != NULL) {
+        password = (const char*)env->GetStringUTFChars(passwordJ, &isCopy);
+        tskIsImage.setFileSystemPassword(string(password));
+        env->ReleaseStringUTFChars(passwordJ, (const char*)password);
+    }
+
+    jstring resultStr = env->NewStringUTF(""); // This will stay empty if we can open the image/file system
+
+    // It seems like passing &imagePathT should work instead of making this new array,
+    // but it generated an EXCEPTION_ACCESS_VIOLATION during testing.
+    TSK_TCHAR ** imagePaths = (TSK_TCHAR**)tsk_malloc((1) * sizeof(TSK_TCHAR*));
+    imagePaths[0] = imagePathT;
+    if (tskIsImage.openImage(1, imagePaths, TSK_IMG_TYPE_DETECT, 0)) {
+        resultStr = env->NewStringUTF("Error opening image");
+    } else {
+        tskIsImage.findFilesInImg();
+        resultStr = env->NewStringUTF(tskIsImage.getMessageForIsImageSupportedNat().c_str());
+    }
+
+    // Cleanup
+    tskIsImage.closeImage();
+    free(imagePaths);
+
+    return resultStr;
 }
 
 /*
