@@ -1,7 +1,7 @@
 /*
- * Autopsy Forensic Browser
+ * Sleuth Kit Data Model
  * 
- * Copyright 2011 Basis Technology Corp.
+ * Copyright 2011-2017 Basis Technology Corp.
  * Contact: carrier <at> sleuthkit <dot> org
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -19,6 +19,12 @@
 package org.sleuthkit.datamodel;
 
 import java.util.List;
+import java.util.Map;
+import org.apache.commons.lang3.ArrayUtils;
+import org.apache.commons.lang3.StringUtils;
+import com.google.gson.Gson;
+import com.google.gson.JsonSyntaxException;
+import static org.sleuthkit.datamodel.SleuthkitCase.IMAGE_PASSWORD_KEY;
 
 /**
  * Represents a file system object stored in tsk_fs_info table FileSystem has a
@@ -38,16 +44,16 @@ public class FileSystem extends AbstractContent {
 	/**
 	 * Constructor most inputs are from the database
 	 *
-	 * @param db the case handle
-	 * @param obj_id the unique object id
-	 * @param name filesystem name
-	 * @param img_offset image offset
-	 * @param fs_type filesystem type
-	 * @param block_size block size in this fs
+	 * @param db          the case handle
+	 * @param obj_id      the unique object id
+	 * @param name        filesystem name
+	 * @param img_offset  image offset
+	 * @param fs_type     filesystem type
+	 * @param block_size  block size in this fs
 	 * @param block_count number of blocks in this fs
-	 * @param root_inum the root inum
-	 * @param first_inum the first inum
-	 * @param last_inum the last inum
+	 * @param root_inum   the root inum
+	 * @param first_inum  the first inum
+	 * @param last_inum   the last inum
 	 */
 	protected FileSystem(SleuthkitCase db, long obj_id, String name, long img_offset,
 			TskData.TSK_FS_TYPE_ENUM fs_type, long block_size, long block_count, long root_inum,
@@ -69,6 +75,10 @@ public class FileSystem extends AbstractContent {
 
 	@Override
 	public int read(byte[] buf, long offset, long len) throws TskCoreException {
+		Content dataSource = getDataSource();
+		if (dataSource instanceof Image && ArrayUtils.isEmpty(((Image) dataSource).getPaths())) {
+			return 0;
+		}
 		return SleuthkitJNI.readFs(getFileSystemHandle(), buf, offset, len);
 	}
 
@@ -82,25 +92,64 @@ public class FileSystem extends AbstractContent {
 	 * this is called and maintains the handle to it to reuse it
 	 *
 	 * @return a filesystem pointer from the sleuthkit
+	 *
 	 * @throws TskCoreException exception throw if an internal tsk core error
-	 * occurs
+	 *                          occurs
 	 */
 	long getFileSystemHandle() throws TskCoreException {
 		if (filesystemHandle == 0) {
 			synchronized (this) {
 				if (filesystemHandle == 0) {
 					Content dataSource = getDataSource();
-					if ((dataSource != null) && (dataSource instanceof Image)) {
-						Image image = (Image) dataSource;
-						filesystemHandle = SleuthkitJNI.openFs(image.getImageHandle(), imgOffset);
-					} else {
+					if ((dataSource == null) || ( !(dataSource instanceof Image))) {
 						throw new TskCoreException("Data Source of File System is not an image");
 					}
+
+					Image image = (Image) dataSource;
+
+					// Check if this file system is in a pool
+					if (isPoolContent()) {
+						Pool pool = getPool();
+						if (pool == null) {
+							throw new TskCoreException("Error finding pool for file system");
+						}
+
+						Volume poolVolume = getPoolVolume();
+						if (poolVolume == null) {
+							throw new TskCoreException("File system is in a pool but has no volume");
+						}
+						filesystemHandle = SleuthkitJNI.openFsPool(image.getImageHandle(), imgOffset, pool.getPoolHandle(), poolVolume.getStart(), getSleuthkitCase());
+					} else {
+						String password = getImagePasswordFromSettings(image.getAcquisitionToolSettings());
+						filesystemHandle = SleuthkitJNI.openFs(image.getImageHandle(), imgOffset, password, getSleuthkitCase());
+					}
 				}
 			}
 		}
 		return this.filesystemHandle;
 	}
+	
+	/**
+	 * Attempt to read the image password from the settings string 
+	 * 
+	 * @param settingsStr
+	 * 
+	 * @return the password if found, empty string otherwise
+	 */
+	private String getImagePasswordFromSettings(String settingsStr) {
+		
+		if(StringUtils.isBlank(settingsStr)){
+			return "";
+		}
+
+		try {
+			Map<String, Object> settingsMap = (new Gson()).fromJson(settingsStr, Map.class);
+			return (String)settingsMap.getOrDefault(IMAGE_PASSWORD_KEY, "");
+		} catch (JsonSyntaxException ex) {
+			// There's no guarantee that acquisition settings will contain a valid JSON string
+			return "";
+		}
+	}
 
 	public Directory getRootDirectory() throws TskCoreException {
 
@@ -179,18 +228,19 @@ public class FileSystem extends AbstractContent {
 		return lastInum;
 	}
 
+	@SuppressWarnings("deprecation")
 	@Override
 	public void finalize() throws Throwable {
 		try {
 			if (filesystemHandle != 0) {
-				SleuthkitJNI.closeFs(filesystemHandle);
+				// SleuthkitJNI.closeFs(filesystemHandle); // closeFs is currently a no-op
 				filesystemHandle = 0;
 			}
 		} finally {
 			super.finalize();
 		}
 	}
-
+	
 	@Override
 	public <T> T accept(SleuthkitItemVisitor<T> v) {
 		return v.visit(this);
@@ -201,16 +251,6 @@ public class FileSystem extends AbstractContent {
 		return v.visit(this);
 	}
 
-	@Override
-	public List<Content> getChildren() throws TskCoreException {
-		return getSleuthkitCase().getAbstractFileChildren(this);
-	}
-
-	@Override
-	public List<Long> getChildrenIds() throws TskCoreException {
-		return getSleuthkitCase().getAbstractFileChildrenIds(this);
-	}
-
 	@Override
 	public String toString(boolean preserveState) {
 		return super.toString(preserveState) + "FileSystem [\t" + " blockCount " + blockCount + "\t" + "blockSize " + blockSize + "\t" + "firstInum " + firstInum + "\t" + "fsType " + fsType + "\t" + "imgOffset " + imgOffset + "\t" + "lastInum " + lastInum + "\t" + "rootInum " + rootInum + "\t" + "]"; //NON-NLS
