@@ -1,6 +1,6 @@
 /*
  ** tsk_recover
- ** The Sleuth Kit 
+ ** The Sleuth Kit
  **
  ** Brian Carrier [carrier <at> sleuthkit [dot] org]
  ** Copyright (c) 2010-2011 Brian Carrier.  All Rights reserved
@@ -21,7 +21,7 @@ usage()
 {
     TFPRINTF(stderr,
         _TSK_T
-        ("usage: %s [-vVae] [-f fstype] [-i imgtype] [-b dev_sector_size] [-o sector_offset] [-P pooltype] [-B pool_volume_block] [-d dir_inum] image [image] output_dir\n"),
+        ("usage: %" PRIttocTSK " [-vVae] [-f fstype] [-i imgtype] [-b dev_sector_size] [-o sector_offset] [-P pooltype] [-B pool_volume_block] [-d dir_inum] image [image] output_dir\n"),
         progname);
     tsk_fprintf(stderr,
         "\t-i imgtype: The format of the image file (use '-i list' for supported types)\n");
@@ -40,8 +40,9 @@ usage()
         "\t-P pooltype: Pool container type (use '-P list' for supported types)\n");
     tsk_fprintf(stderr,
         "\t-B pool_volume_block: Starting block (for pool volumes only)\n");
-    tsk_fprintf(stderr, 
+    tsk_fprintf(stderr,
         "\t-d dir_inum: Directory inum to recover from (must also specify a specific partition using -o or there must not be a volume system)\n");
+    tsk_fprintf(stderr, "\t-k password: Decryption password for encrypted volumes\n");
 
     exit(1);
 }
@@ -58,10 +59,10 @@ public:
     virtual TSK_RETVAL_ENUM processFile(TSK_FS_FILE * fs_file, const char *path);
     virtual TSK_FILTER_ENUM filterVol(const TSK_VS_PART_INFO * vs_part);
     virtual TSK_FILTER_ENUM filterFs(TSK_FS_INFO * fs_info);
-    uint8_t openFs(TSK_OFF_T a_soffset, TSK_FS_TYPE_ENUM fstype, TSK_POOL_TYPE_ENUM pooltype, TSK_DADDR_T pvol_block);
+    uint8_t openFs(TSK_OFF_T a_soffset, TSK_FS_TYPE_ENUM fstype, TSK_POOL_TYPE_ENUM pooltype, TSK_DADDR_T pvol_block, const char* password);
     uint8_t findFiles(TSK_INUM_T a_dirInum);
     uint8_t handleError();
-    
+
 private:
     TSK_TCHAR * m_base_dir;
     uint8_t writeFile(TSK_FS_FILE * a_fs_file, const char *a_path);
@@ -85,19 +86,24 @@ TskRecover::TskRecover(TSK_TCHAR * a_base_dir)
 }
 
 // Print errors as they are encountered
-uint8_t TskRecover::handleError() 
+uint8_t TskRecover::handleError()
 {
     fprintf(stderr, "%s", tsk_error_get());
     return 0;
-} 
+}
 
 /** \internal
  * Callback used to walk file content and write the results to the recovery file.
  */
 static TSK_WALK_RET_ENUM
-file_walk_cb(TSK_FS_FILE * a_fs_file, TSK_OFF_T a_off,
-    TSK_DADDR_T a_addr, char *a_buf, size_t a_len,
-    TSK_FS_BLOCK_FLAG_ENUM a_flags, void *a_ptr)
+file_walk_cb(
+  [[maybe_unused]] TSK_FS_FILE * a_fs_file,
+  [[maybe_unused]] TSK_OFF_T a_off,
+  [[maybe_unused]] TSK_DADDR_T a_addr,
+  char *a_buf,
+  size_t a_len,
+  [[maybe_unused]] TSK_FS_BLOCK_FLAG_ENUM a_flags,
+  void *a_ptr)
 {
     //write to the file
 #ifdef TSK_WIN32
@@ -123,17 +129,17 @@ file_walk_cb(TSK_FS_FILE * a_fs_file, TSK_OFF_T a_off,
  */
 uint8_t TskRecover::writeFile(TSK_FS_FILE * a_fs_file, const char *a_path)
 {
-    
+
 #ifdef TSK_WIN32
-    /* Step 1 is to make the full path in UTF-16 and create the 
+    /* Step 1 is to make the full path in UTF-16 and create the
      * needed directories. */
-    
+
     // combine the volume name and path
     char path8[FILENAME_MAX];
     strncpy(path8, m_vsName, FILENAME_MAX);
-    strncat(path8, a_path, FILENAME_MAX-strlen(path8)); 
+    strncat(path8, a_path, FILENAME_MAX-strlen(path8));
     size_t ilen = strlen(path8);
-    
+
     // clean up any control characters
     for (size_t i = 0; i < ilen; i++) {
         if (TSK_IS_CNTRL(path8[i]))
@@ -181,7 +187,7 @@ uint8_t TskRecover::writeFile(TSK_FS_FILE * a_fs_file, const char *a_path)
                 result = CreateDirectoryW((LPCTSTR) path16full, NULL);
             if (result == FALSE) {
                 if (GetLastError() == ERROR_PATH_NOT_FOUND) {
-                    fprintf(stderr, "Error Creating Directory (%S)", path16full);
+                    fprintf(stderr, "Error Creating Directory (%ls)", path16full);
                     return 1;
                 }
             }
@@ -195,7 +201,7 @@ uint8_t TskRecover::writeFile(TSK_FS_FILE * a_fs_file, const char *a_path)
         path16full[len] = L'\\';
 
     //do name mangling
-    char name8[FILENAME_MAX];
+    char name8[FILENAME_MAX + 1];
     strncpy(name8, a_fs_file->name->name, FILENAME_MAX);
     for (int i = 0; name8[i] != '\0'; i++) {
         if (TSK_IS_CNTRL(name8[i]))
@@ -203,14 +209,14 @@ uint8_t TskRecover::writeFile(TSK_FS_FILE * a_fs_file, const char *a_path)
     }
 
     //convert file name from utf8 to utf16
-    wchar_t name16[FILENAME_MAX];
+    wchar_t name16[FILENAME_MAX + 1];
 
     ilen = strlen(name8);
     utf8 = (UTF8 *) name8;
     utf16 = (UTF16 *) name16;
 
     retVal = tsk_UTF8toUTF16((const UTF8 **) &utf8, &utf8[ilen],
-        &utf16, &utf16[FILENAME_MAX], TSKlenientConversion);
+        &utf16, &utf16[FILENAME_MAX + 1], TSKlenientConversion);
     *utf16 = '\0';
 
     if (retVal != TSKconversionOK) {
@@ -219,7 +225,7 @@ uint8_t TskRecover::writeFile(TSK_FS_FILE * a_fs_file, const char *a_path)
     }
 
     //append the file name onto the path
-    wcsncat(path16full, name16, FILENAME_MAX-wcslen(path16full));
+    wcsncat(path16full, name16, FILENAME_MAX + 1 - wcslen(path16full));
 
     //create the file
     HANDLE
@@ -227,14 +233,14 @@ uint8_t TskRecover::writeFile(TSK_FS_FILE * a_fs_file, const char *a_path)
         CreateFileW((LPCTSTR) path16full, GENERIC_WRITE, 0, NULL, OPEN_ALWAYS,
         FILE_ATTRIBUTE_NORMAL, NULL);
     if (handle == INVALID_HANDLE_VALUE) {
-        fprintf(stderr, "Error Creating File (%S)", path16full);
+        fprintf(stderr, "Error Creating File (%ls)", path16full);
         return 1;
     }
 
     //try to write to the file
     if (tsk_fs_file_walk(a_fs_file, (TSK_FS_FILE_WALK_FLAG_ENUM) 0,
             file_walk_cb, handle)) {
-        fprintf(stderr, "Error writing file %S\n", path16full);
+        fprintf(stderr, "Error writing file %ls\n", path16full);
         tsk_error_print(stderr);
         CloseHandle(handle);
         return 1;
@@ -250,15 +256,18 @@ uint8_t TskRecover::writeFile(TSK_FS_FILE * a_fs_file, const char *a_path)
     FILE *
         hFile;
 
-    snprintf(fbuf, PATH_MAX, "%s/%s/%s", (char *) m_base_dir, m_vsName,
-        a_path);
+    const int ret = snprintf(fbuf, PATH_MAX, "%s/%s/%s", (char *) m_base_dir, m_vsName, a_path);
+    if (ret >= PATH_MAX) {
+        fprintf(stderr, "Error: path '%s/%s/%s' truncated\n", (char *) m_base_dir, m_vsName, a_path);
+        return 1;
+    }
 
     // clean up any control characters in path
     for (size_t i = 0; i < strlen(fbuf); i++) {
         if (TSK_IS_CNTRL(fbuf[i]))
             fbuf[i] = '^';
     }
-    
+
     // see if the directory already exists. Create, if not.
     if (0 != lstat(fbuf, &statds)) {
         size_t
@@ -291,7 +300,7 @@ uint8_t TskRecover::writeFile(TSK_FS_FILE * a_fs_file, const char *a_path)
         strncat(fbuf, "/", PATH_MAX - strlen(fbuf)-1);
 
     strncat(fbuf, a_fs_file->name->name, PATH_MAX - strlen(fbuf)-1);
-    
+
     //do name mangling of the file name that was just added
     for (int i = strlen(fbuf)-1; fbuf[i] != '/'; i--) {
         if (TSK_IS_CNTRL(fbuf[i]))
@@ -365,11 +374,11 @@ TskRecover::filterFs(TSK_FS_INFO * fs_info)
 }
 
 uint8_t
-TskRecover::openFs(TSK_OFF_T a_soffset, TSK_FS_TYPE_ENUM fstype, TSK_POOL_TYPE_ENUM pooltype, TSK_DADDR_T pvol_block)
+TskRecover::openFs(TSK_OFF_T a_soffset, TSK_FS_TYPE_ENUM fstype, TSK_POOL_TYPE_ENUM pooltype, TSK_DADDR_T pvol_block, const char* password)
 {
     if (pvol_block == 0) {
         if ((m_fs_info = tsk_fs_open_img_decrypt(m_img_info, a_soffset * m_img_info->sector_size,
-            fstype, "")) == NULL) {
+            fstype, password)) == NULL) {
             tsk_error_print(stderr);
             if (tsk_error_get_errno() == TSK_ERR_FS_UNSUPTYPE)
                 tsk_fs_type_print(stderr);
@@ -386,7 +395,7 @@ TskRecover::openFs(TSK_OFF_T a_soffset, TSK_FS_TYPE_ENUM fstype, TSK_POOL_TYPE_E
         }
 
         m_img_info = pool->get_img_info(pool, pvol_block);
-        if ((m_fs_info = tsk_fs_open_img_decrypt(m_img_info, a_soffset * m_img_info->sector_size, fstype, "")) == NULL) {
+        if ((m_fs_info = tsk_fs_open_img_decrypt(m_img_info, a_soffset * m_img_info->sector_size, fstype, password)) == NULL) {
             tsk_error_print(stderr);
             if (tsk_error_get_errno() == TSK_ERR_FS_UNSUPTYPE)
                 tsk_fs_type_print(stderr);
@@ -423,6 +432,7 @@ main(int argc, char **argv1)
     TSK_TCHAR *cp;
     TSK_FS_DIR_WALK_FLAG_ENUM walkflag = TSK_FS_DIR_WALK_FLAG_UNALLOC;
     TSK_INUM_T dirInum = 0;
+    const char* password = "";
 
 #ifdef TSK_WIN32
     // On Windows, get the wide arguments (mingw doesn't support wmain)
@@ -438,33 +448,34 @@ main(int argc, char **argv1)
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = GETOPT(argc, argv, _TSK_T("ab:B:d:ef:i:o:P:vV"))) > 0) {
+    while ((ch = GETOPT(argc, argv, _TSK_T("ab:B:d:ef:i:k:o:P:vV"))) > 0) {
         switch (ch) {
         case _TSK_T('?'):
         default:
-            TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
+            TFPRINTF(stderr, _TSK_T("Invalid argument: %" PRIttocTSK "\n"),
                 argv[OPTIND]);
             usage();
+            break;
 
         case _TSK_T('a'):
             walkflag = TSK_FS_DIR_WALK_FLAG_ALLOC;
             break;
-            
+
         case _TSK_T('b'):
             ssize = (unsigned int) TSTRTOUL(OPTARG, &cp, 0);
             if (*cp || *cp == *OPTARG || ssize < 1) {
                 TFPRINTF(stderr,
                     _TSK_T
-                    ("invalid argument: sector size must be positive: %s\n"),
+                    ("invalid argument: sector size must be positive: %" PRIttocTSK "\n"),
                     OPTARG);
                 usage();
             }
             break;
-                
+
         case _TSK_T('d'):
             if (tsk_fs_parse_inum(OPTARG, &dirInum, NULL, NULL, NULL, NULL)) {
                 TFPRINTF(stderr,
-                        _TSK_T("invalid argument for directory inode: %s\n"),
+                        _TSK_T("invalid argument for directory inode: %" PRIttocTSK "\n"),
                         OPTARG);
                 usage();
             }
@@ -475,7 +486,7 @@ main(int argc, char **argv1)
             (TSK_FS_DIR_WALK_FLAG_ENUM) (TSK_FS_DIR_WALK_FLAG_UNALLOC |
                                          TSK_FS_DIR_WALK_FLAG_ALLOC);
             break;
-                
+
         case _TSK_T('f'):
             if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
                 tsk_fs_type_print(stderr);
@@ -484,7 +495,7 @@ main(int argc, char **argv1)
             fstype = tsk_fs_type_toid(OPTARG);
             if (fstype == TSK_FS_TYPE_UNSUPP) {
                 TFPRINTF(stderr,
-                         _TSK_T("Unsupported file system type: %s\n"), OPTARG);
+                         _TSK_T("Unsupported file system type: %" PRIttocTSK "\n"), OPTARG);
                 usage();
             }
             break;
@@ -497,12 +508,12 @@ main(int argc, char **argv1)
             }
             imgtype = tsk_img_type_toid(OPTARG);
             if (imgtype == TSK_IMG_TYPE_UNSUPP) {
-                TFPRINTF(stderr, _TSK_T("Unsupported image type: %s\n"),
+                TFPRINTF(stderr, _TSK_T("Unsupported image type: %" PRIttocTSK "\n"),
                     OPTARG);
                 usage();
             }
             break;
-                
+
         case _TSK_T('o'):
             if ((soffset = tsk_parse_offset(OPTARG)) == -1) {
                 tsk_error_print(stderr);
@@ -530,6 +541,10 @@ main(int argc, char **argv1)
             }
             break;
 
+        case _TSK_T('k'):
+            password = argv1[OPTIND - 1];
+            break;
+
         case _TSK_T('v'):
             tsk_verbose++;
             break;
@@ -548,15 +563,16 @@ main(int argc, char **argv1)
     }
 
     TskRecover tskRecover(argv[argc-1]);
+    tskRecover.setFileSystemPassword(password);
 
-    tskRecover.setFileFilterFlags(walkflag);    
+    tskRecover.setFileFilterFlags(walkflag);
     if (tskRecover.openImage(argc - OPTIND - 1, &argv[OPTIND], imgtype,
             ssize)) {
         tsk_error_print(stderr);
         exit(1);
     }
-    
-    if (tskRecover.openFs(soffset, fstype, pooltype, (TSK_DADDR_T)pvol_block)) {
+
+    if (tskRecover.openFs(soffset, fstype, pooltype, (TSK_DADDR_T)pvol_block, password)) {
         // Errors were already logged
         exit(1);
     }
