@@ -19,7 +19,9 @@
 
 /* config.h must be first */
 #include "tsk/tsk_tools_i.h"
+
 #include "fiwalk.h"
+#include "content.h"
 #include "arff.h"
 #include "plugin.h"
 #include "unicode_escape.h"
@@ -74,14 +76,13 @@ file_act(TSK_FS_FILE * fs_file, TSK_OFF_T a_off, TSK_DADDR_T addr, char *buf,
  * See also tsk_fs_name_print() in ./tsk/fs/fs_name.c
  */
 
-static uint8_t
-process_tsk_file(TSK_FS_FILE * fs_file, const char *path)
+uint8_t fiwalk::process_tsk_file(TSK_FS_FILE * fs_file, const char *path)
 {
     /* Use a flag to determine if a file is generically fit for plugins. */
     bool can_run_plugin;
 
     /* Make sure that the SleuthKit structures are properly set */
-    if (fs_file->name == NULL) 
+    if (fs_file->name == NULL)
         return 1;
     if (fs_file->meta == NULL && opt_debug)
         printf("File: %s %s  has no meta\n", path, fs_file->name->name);
@@ -93,7 +94,7 @@ process_tsk_file(TSK_FS_FILE * fs_file, const char *path)
 			  tsk_fs_name_type_str[fs_file->name->type],fs_file->name->type);
 
     /* Recover the filename from the fs_dent, if it is provided */
-    content ci(fs_file->fs_info->img_info);	// where the content will go
+    content ci(fs_file->fs_info->img_info, *this);	// where the content will go
     ci.evidence_dirname = path;
     ci.set_filename(fs_file->name->name);
 
@@ -101,47 +102,43 @@ process_tsk_file(TSK_FS_FILE * fs_file, const char *path)
     if (ci.name_filtered()) return 0;
 
     /* Looks like we are processing */
-    if(a) a->new_row();			// tell ARFF we are starting a new row
-    if(x) x->push("fileobject"); 	// tell XML we are starting a new XML object
-    if(opt_parent_tracking)
-    {
-        if(fs_file->name->par_addr){
-            if(x)
-            {
+    if (a) a->new_row();			// tell ARFF we are starting a new row
+    if (x) x->push("fileobject"); 	// tell XML we are starting a new XML object
+    if (opt_parent_tracking) {
+        if (fs_file->name->par_addr){
+            if (x) {
                 x->push("parent_object");
                 file_info("inode", fs_file->name->par_addr);
                 if(x) x->pop();
             }
-            if((t||a) && !opt_body_file)
-            {
+            if ((t||a) && !opt_body_file) {
                 file_info("parent_inode", fs_file->name->par_addr);
             }
         }
     }
 
-    if(fs_file->meta != NULL)
-    {
+    if(fs_file->meta != NULL) {
         /* Get the content if needed */
         if(ci.need_file_walk() && (opt_maxgig==0 || fs_file->meta->size/1000000000 < opt_maxgig)){
-    	int myflags = TSK_FS_FILE_WALK_FLAG_NOID;
-    	if (opt_no_data) myflags |= TSK_FS_FILE_WALK_FLAG_AONLY;
-    	if (tsk_fs_file_walk (fs_file, (TSK_FS_FILE_WALK_FLAG_ENUM) myflags, file_act, (void *) &ci)) {
-    
-    	    // ignore errors from deleted files that were being recovered
-    	    //if (tsk_errno != TSK_ERR_FS_RECOVER) {
-    	    if (tsk_error_get_errno() != TSK_ERR_FS_RECOVER) {
-    		if(opt_debug){
-    		    fprintf(stderr,"Processing: %s/%s (%" PRIuINUM ")\n", path,
-    			   fs_file->name->name, fs_file->meta->addr);
-    		    tsk_error_print(stderr);
-    		}
-    	    }
-    	    tsk_error_reset();
-    	}
+            int myflags = TSK_FS_FILE_WALK_FLAG_NOID;
+            if (opt_no_data) myflags |= TSK_FS_FILE_WALK_FLAG_AONLY;
+            if (tsk_fs_file_walk (fs_file, (TSK_FS_FILE_WALK_FLAG_ENUM) myflags, file_act, (void *) &ci)) {
+
+                // ignore errors from deleted files that were being recovered
+                //if (tsk_errno != TSK_ERR_FS_RECOVER) {
+                if (tsk_error_get_errno() != TSK_ERR_FS_RECOVER) {
+                    if (opt_debug){
+                        fprintf(stderr,"Processing: %s/%s (%" PRIuINUM ")\n", path,
+                                fs_file->name->name, fs_file->meta->addr);
+                        tsk_error_print(stderr);
+                    }
+                }
+                tsk_error_reset();
+            }
         }
     }
 
-    if(file_count_max && file_count>file_count_max) return TSK_WALK_STOP;
+    if (file_count_max && file_count>file_count_max) return TSK_WALK_STOP;
     file_count++;
 
     /* Send through to the plugin if we were doing that.
@@ -149,11 +146,11 @@ process_tsk_file(TSK_FS_FILE * fs_file, const char *path)
      */
 
     /* Finally output the informaton */
-    if(opt_body_file && (fs_file->meta != NULL)){
+    if (opt_body_file && (fs_file->meta != NULL)){
 	char ls[64];
 	tsk_fs_meta_make_ls(fs_file->meta,ls,sizeof(ls));
-	fprintf(t,"%s|%s|%"PRId64"|%s|%d|%d|%"PRId64"|%d|%d|%d|%d\n",
-		ci.h_md5.final().hexdigest().c_str(),ci.filename().c_str(),fs_file->meta->addr,
+	fprintf(t,"%s|%s|%" PRId64 "|%s|%d|%d|%" PRId64 "|%d|%d|%d|%d\n",
+		ci.h_md5.finalize().hexdigest().c_str(),ci.filename().c_str(),fs_file->meta->addr,
 		ls,fs_file->meta->uid,fs_file->meta->gid,
 		fs_file->meta->size,
 		(uint32_t)(fs_file->meta->atime),
@@ -173,28 +170,52 @@ process_tsk_file(TSK_FS_FILE * fs_file, const char *path)
     /* fs_file->name */
 
     if(ci.has_filename()) file_info("filename",validateOrEscapeUTF8(ci.filename()));
-    file_info("partition",current_partition_num);
-    file_info("id",next_id++);
+    file_info("partition", current_partition_num);
+    file_info("id", next_id++);
     file_info("name_type",tsk_fs_name_type_str[fs_file->name->type]);
 
+    /* Report metadata structures' allocation first */
+    if (fs_file->meta != NULL)
+    {
+        // Also report filesize to preserve the original element order, and save an if branch
+        file_info("filesize",fs_file->meta->size);
+
+        if(fs_file->meta->flags & TSK_FS_META_FLAG_ALLOC)   file_info("alloc_inode",1);
+        if(fs_file->meta->flags & TSK_FS_META_FLAG_UNALLOC) file_info("alloc_inode",0);
+    }
+    if(fs_file->name != NULL)
+    {
+        if(fs_file->name->flags & TSK_FS_NAME_FLAG_ALLOC)   file_info("alloc_name",1);
+        if(fs_file->name->flags & TSK_FS_NAME_FLAG_UNALLOC) file_info("alloc_name",0);
+    }
+
+
+    /* Report contents of metadata structures */
     if(fs_file->meta != NULL)
     {
         /* fs_file->meta */
-        file_info("filesize",fs_file->meta->size);
-        if(fs_file->meta->flags & TSK_FS_META_FLAG_ALLOC)   file_info("alloc",1);
-        if(fs_file->meta->flags & TSK_FS_META_FLAG_UNALLOC) file_info("unalloc",1);
         if(fs_file->meta->flags & TSK_FS_META_FLAG_USED)    file_info("used",1);
         if(fs_file->meta->flags & TSK_FS_META_FLAG_UNUSED)  file_info("unused",1);
         if(fs_file->meta->flags & TSK_FS_META_FLAG_ORPHAN)  file_info("orphan",1);
         if(fs_file->meta->flags & TSK_FS_META_FLAG_COMP)    file_info("compressed",1);
-    
+
         file_info("inode",fs_file->meta->addr);
         file_info("meta_type",fs_file->meta->type);
         file_info("mode",fs_file->meta->mode); // *** REPLACE WITH drwx-rw-rw or whatever
         file_info("nlink",fs_file->meta->nlink);
         file_info("uid",fs_file->meta->uid);
         file_info("gid",fs_file->meta->gid);
-    
+
+        char i_buf[1024];
+        memset(i_buf, 0, 1024);
+        string i_runs = "";
+        uint64_t current_partition_start = fs_file->fs_info->offset;
+        if (fs_file->meta->start_of_inode != 0){
+            snprintf(i_buf,sizeof(i_buf),"       <byte_run fs_offset='%" PRIu64 "' img_offset='%" PRIu64 "'/>\n",fs_file->meta->start_of_inode, current_partition_start + fs_file->meta->start_of_inode);
+        }
+        i_runs += i_buf;
+        file_info_xml2("byte_runs","facet='inode'", i_runs);
+
     	/* Special processing for FAT */
     	if(TSK_FS_TYPE_ISFAT(fs_file->fs_info->ftype))
     	{
@@ -209,17 +230,17 @@ process_tsk_file(TSK_FS_FILE * fs_file, const char *path)
            if(fs_file->meta->atime) file_infot("atime",fs_file->meta->atime);
            if(fs_file->meta->crtime) file_infot("crtime",fs_file->meta->crtime);
     	}
-    
+
         /* TK: do content_ptr */
         if(fs_file->meta->seq!=0) file_info("seq",fs_file->meta->seq);
-    
+
         /* Special processing for EXT */
         if(TSK_FS_TYPE_ISEXT(fs_file->fs_info->ftype)){
     	if(fs_file->meta->time2.ext2.dtime){
     	    file_infot("dtime",fs_file->meta->time2.ext2.dtime);
     	}
         }
-    
+
         /* Special processing for HFS */
         if(TSK_FS_TYPE_ISHFS(fs_file->fs_info->ftype)){
     	if(fs_file->meta->time2.hfs.bkup_time){
@@ -227,20 +248,18 @@ process_tsk_file(TSK_FS_FILE * fs_file, const char *path)
     	}
         }
     }
-    if(fs_file->meta == NULL)
-    {
-        if(fs_file->name->flags & TSK_FS_META_FLAG_ALLOC)   file_info("alloc",1);
-        if(fs_file->name->flags & TSK_FS_META_FLAG_UNALLOC) file_info("unalloc",1);
-        if(fs_file->name->flags & TSK_FS_META_FLAG_USED)    file_info("used",1);
-        if(fs_file->name->flags & TSK_FS_META_FLAG_UNUSED)  file_info("unused",1);
-        if(fs_file->name->flags & TSK_FS_META_FLAG_ORPHAN)  file_info("orphan",1);
-        if(fs_file->name->flags & TSK_FS_META_FLAG_COMP)    file_info("compressed",1);
-    
+
+    // fs_file->meta == NULL)
+    else {
+
+        // @@@ BC: This is a bit confusing.  It seems to be cramming NAME-level info
+        // into places that typically has META-level info.
         if (fs_file->name->meta_addr!=0)file_info("inode",fs_file->name->meta_addr);
         file_info("meta_type",fs_file->name->type);
-        
+
         if(fs_file->name->meta_seq!=0) file_info("seq",fs_file->name->meta_seq);
     }
+
     /* Special processing for NTFS */
     if ((TSK_FS_TYPE_ISNTFS(fs_file->fs_info->ftype))){
 	/* Should we cycle through the attributes the way print_dent_act() does? */
@@ -292,16 +311,17 @@ process_tsk_file(TSK_FS_FILE * fs_file, const char *path)
  * that is found.
  */
 static TSK_WALK_RET_ENUM
-dir_act(TSK_FS_FILE * fs_file, const char *path, void *ptr)
+dir_act(TSK_FS_FILE * fs_file, const char *path, void * that)
 {
     /* Ignore NTFS System files */
-    if (opt_ignore_ntfs_system_files
+    fiwalk *o = (fiwalk *)that;
+    if ( o->opt_ignore_ntfs_system_files
 	&& (TSK_FS_TYPE_ISNTFS(fs_file->fs_info->ftype) || TSK_FS_TYPE_ISFAT(fs_file->fs_info->ftype))
         && (fs_file->name->name[0] == '$'))
         return TSK_WALK_CONT;
 
     /* If the name has corresponding metadata, then walk it */
-   	process_tsk_file(fs_file, path);
+    o->process_tsk_file(fs_file, path);
 
     return TSK_WALK_CONT;
 }
@@ -314,15 +334,16 @@ dir_act(TSK_FS_FILE * fs_file, const char *path, void *ptr)
  * @return 0 if success, -1 if fail
  */
 
-int proc_fs(TSK_IMG_INFO * img_info, TSK_OFF_T start)
+
+int fiwalk::proc_fs(TSK_IMG_INFO * img_info, TSK_OFF_T start)
 {
     TSK_FS_INFO *fs_info;
-    u_int sector_size = img_info->sector_size;
+    uint32_t sector_size = img_info->sector_size;
 
     /* Try it as a file system */
     fs_info = tsk_fs_open_img(img_info, start, TSK_FS_TYPE_DETECT);
     if (fs_info == NULL) {
-	comment("TSK_Error '%s' at sector %"PRIuDADDR" offset %"PRIuDADDR" sector_size=%u",
+	comment("TSK_Error '%s' at sector %" PRIuDADDR " offset %" PRIuDADDR " sector_size=%u",
 		tsk_error_get(),start/sector_size,start,sector_size);
 
 	/* We could do some carving on the volume data at this point */
@@ -332,7 +353,7 @@ int proc_fs(TSK_IMG_INFO * img_info, TSK_OFF_T start)
     comment("fs start: %" PRIuDADDR, start);
     if(x){
 	char buf[1024];
-	snprintf(buf,sizeof(buf),"offset='%"PRIuDADDR"'",start);
+	snprintf(buf,sizeof(buf),"offset='%" PRIuDADDR "'",start);
 	x->push("volume",buf);
     }
 
@@ -361,7 +382,7 @@ int proc_fs(TSK_IMG_INFO * img_info, TSK_OFF_T start)
     /* Walk the files, starting at the root directory */
     int dir_walk_flags = TSK_FS_DIR_WALK_FLAG_RECURSE | TSK_FS_DIR_WALK_FLAG_ALLOC;
 
-    if(opt_allocated_only){
+    if (opt_allocated_only){
 	dir_walk_flags |= TSK_FS_DIR_WALK_FLAG_NOORPHAN;
 	partition_info("allocated_only",1);
     }
@@ -371,7 +392,7 @@ int proc_fs(TSK_IMG_INFO * img_info, TSK_OFF_T start)
 
     int ret = 0;
     if (tsk_fs_dir_walk(fs_info, fs_info->root_inum,
-			(TSK_FS_DIR_WALK_FLAG_ENUM) dir_walk_flags, dir_act, NULL)) {
+                        (TSK_FS_DIR_WALK_FLAG_ENUM) dir_walk_flags, dir_act, this)) {
 	comment("TSK Error: tsk_fs_dir_walk: ",tsk_error_get());
 	ret = -1;
     }
@@ -389,10 +410,10 @@ int proc_fs(TSK_IMG_INFO * img_info, TSK_OFF_T start)
  * each volume to find a file system.
  */
 static TSK_WALK_RET_ENUM
-vs_act(TSK_VS_INFO * vs_info, const TSK_VS_PART_INFO * vs_part, void *ptr)
+vs_act(TSK_VS_INFO * vs_info, const TSK_VS_PART_INFO * vs_part, void *that)
 {
-    int *count = (int *)ptr;
-    if (proc_fs(vs_info->img_info, vs_part->start * vs_info->block_size)) {
+    fiwalk *o = (fiwalk *)that;
+    if (o->proc_fs(vs_info->img_info, vs_part->start * vs_info->block_size)) {
         // if we return ERROR here, then the walk will stop.  But, the
         // error could just be because we looked into an unallocated volume.
         // do any special error handling / reporting here.
@@ -400,7 +421,7 @@ vs_act(TSK_VS_INFO * vs_info, const TSK_VS_PART_INFO * vs_part, void *ptr)
         return TSK_WALK_CONT;
     }
 
-    (*count)++;
+    o->vs_count++;
     return TSK_WALK_CONT;
 }
 
@@ -416,11 +437,10 @@ vs_act(TSK_VS_INFO * vs_info, const TSK_VS_PART_INFO * vs_part, void *ptr)
  * @return -1 on error and 0 on success but nothing found, otherwise the number of file systems
  *             processed without error.
  */
-int proc_vs(TSK_IMG_INFO * img_info)
+int fiwalk::proc_vs(TSK_IMG_INFO * img_info)
 {
     TSK_VS_INFO *vs_info;
     int start = 0;
-    int count = 0;
 
     // USE mm_walk to get the volumes
     if ((vs_info = tsk_vs_open(img_info, start, TSK_VS_TYPE_DETECT)) == NULL) {
@@ -447,16 +467,16 @@ int proc_vs(TSK_IMG_INFO * img_info)
 
         /* Walk the allocated volumes (skip metadata and unallocated volumes) */
         if (tsk_vs_part_walk(vs_info, 0, vs_info->part_count-1,
-                (TSK_VS_PART_FLAG_ENUM) (TSK_VS_PART_FLAG_ALLOC), vs_act, &count)) {
+                             (TSK_VS_PART_FLAG_ENUM) (TSK_VS_PART_FLAG_ALLOC), vs_act, (void *)this)) {
             tsk_vs_close(vs_info);
             return -1;
         }
         tsk_vs_close(vs_info);
     }
-    return count;
+    return vs_count;
 }
 
-void process_scalpel_audit_file(TSK_IMG_INFO *img_info,const char *audit_file)
+void fiwalk::process_scalpel_audit_file(TSK_IMG_INFO *img_info,const char *audit_file)
 {
     const char *fmt = "%12s%qd\t\t%3s%zd";
     FILE *f = fopen(audit_file,"r");
@@ -475,7 +495,7 @@ void process_scalpel_audit_file(TSK_IMG_INFO *img_info,const char *audit_file)
 		if(a) a->new_row();
 		if(x) x->push("fileobject");
 
-		content ci(img_info);
+		content ci(img_info,*this);
 		ci.evidence_dirname = "?/";
 		ci.set_filename(filename); // fictitious filename, but needed for plugins
 
@@ -501,11 +521,13 @@ void process_scalpel_audit_file(TSK_IMG_INFO *img_info,const char *audit_file)
     }
 }
 
-int process_image_file(int argc,char * const *argv,const char *audit_file,u_int sector_size)
+int fiwalk::process_image_file(int argc,char * const *argv,const char *audit_file,uint32_t sector_size)
 {
     TSK_IMG_INFO *img_info;
     int count = 0;
 
+    assert(argc!=0);
+
     img_info = tsk_img_open_utf8(argc,(const char **)argv, TSK_IMG_TYPE_DETECT,sector_size);
 
     if (img_info==0){
@@ -516,8 +538,6 @@ int process_image_file(int argc,char * const *argv,const char *audit_file,u_int
 	    process_scalpel_audit_file(img_info,audit_file);
 	}
 	else{
-	    if (opt_debug) printf("calling do_dimage()\n");
-
 	    int r = proc_vs(img_info);
 	    if (r<0){
 		comment("TSK Error (do_dimage) %s",tsk_error_get());
