@@ -1,9 +1,9 @@
 /*
  * img_cat
- * The Sleuth Kit 
+ * The Sleuth Kit
  *
  * Brian Carrier [carrier <at> sleuthkit [dot] org]
- * Copyright (c) 2006-2011 Brian Carrier, Basis Technology.  All rights reserved 
+ * Copyright (c) 2006-2011 Brian Carrier, Basis Technology.  All rights reserved
  *
  * This software is distributed under the Common Public License 1.0
  */
@@ -21,7 +21,7 @@ usage()
 {
     TFPRINTF(stderr,
         _TSK_T
-        ("usage: %s [-vV] [-i imgtype] [-b dev_sector_size] [-s start_sector] [-e stop_sector] image\n"),
+        ("usage: %" PRIttocTSK " [-vV] [-i imgtype] [-b dev_sector_size] [-s start_sector] [-e stop_sector] image\n"),
         progname);
     tsk_fprintf(stderr,
         "\t-i imgtype: The format of the image file (use 'i list' for supported types)\n");
@@ -68,15 +68,16 @@ main(int argc, char **argv1)
         switch (ch) {
         case _TSK_T('?'):
         default:
-            TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
+            TFPRINTF(stderr, _TSK_T("Invalid argument: %" PRIttocTSK "\n"),
                 argv[OPTIND]);
             usage();
+            break;
         case _TSK_T('b'):
             ssize = (unsigned int) TSTRTOUL(OPTARG, &cp, 0);
             if (*cp || *cp == *OPTARG || ssize < 1) {
                 TFPRINTF(stderr,
                     _TSK_T
-                    ("invalid argument: sector size must be positive: %s\n"),
+                    ("invalid argument: sector size must be positive: %" PRIttocTSK "\n"),
                     OPTARG);
                 usage();
             }
@@ -88,7 +89,7 @@ main(int argc, char **argv1)
             }
             imgtype = tsk_img_type_toid(OPTARG);
             if (imgtype == TSK_IMG_TYPE_UNSUPP) {
-                TFPRINTF(stderr, _TSK_T("Unsupported image type: %s\n"),
+                TFPRINTF(stderr, _TSK_T("Unsupported image type: %" PRIttocTSK "\n"),
                     OPTARG);
                 usage();
             }
@@ -99,7 +100,7 @@ main(int argc, char **argv1)
             if (*cp || *cp == *OPTARG || start_sector < 1) {
                 TFPRINTF(stderr,
                     _TSK_T
-                    ("invalid argument: start sector must be positive: %s\n"),
+                    ("invalid argument: start sector must be positive: %" PRIttocTSK "\n"),
                     OPTARG);
                 usage();
             }
@@ -110,7 +111,7 @@ main(int argc, char **argv1)
             if (*cp || *cp == *OPTARG || end_sector < 1) {
                 TFPRINTF(stderr,
                     _TSK_T
-                    ("invalid argument: end sector must be positive: %s\n"),
+                    ("invalid argument: end sector must be positive: %" PRIttocTSK "\n"),
                     OPTARG);
                 usage();
             }
