@@ -7,9 +7,9 @@
  * This software is distributed under the Common Public License 1.0
  */
 
-/** \file tsk_printf.c 
+/** \file tsk_printf.c
  * These are printf wrappers that are needed so that we can
- * easily print in both Unix and Windows.  For Unix, the 
+ * easily print in both Unix and Windows.  For Unix, the
  * internal UTF-8 representation is kept and a normal printf
  * is performed.  For Windows, the UTF-8 representation is first
  * converted to UTF-16 and then printed
@@ -20,7 +20,7 @@
 
 
 /** \internal
- * Convert the UTF-8 printf arguments to UTF-16 and fill in the 
+ * Convert the UTF-8 printf arguments to UTF-16 and fill in the
  * printf types (%s, %d, etc.)
  *
  * @param wbuf wide char string to write result to
@@ -46,7 +46,7 @@ tsk_printf_conv(WCHAR * wbuf, int wlen, const char *msg, va_list * args)
     if (NULL == (cbuf = (char *) tsk_malloc(clen))) {
         return 1;
     }
-    
+
 #ifdef _MSC_VER
     vsnprintf_s(cbuf, clen - 1, _TRUNCATE, msg, *args);
 #else
@@ -77,10 +77,10 @@ tsk_printf_conv(WCHAR * wbuf, int wlen, const char *msg, va_list * args)
  * fprintf wrapper function that takes UTF-8 strings as input
  * (on all platforms) and does what is necessary to output
  * strings in the correct encoding (UTF-8 on Unix and
- * UTF-16 on Windows). 
+ * UTF-16 on Windows).
  *
  * @param fd File to print to
- * @param msg printf message 
+ * @param msg printf message
  */
 void
 tsk_fprintf(FILE * fd, const char *msg, ...)
@@ -92,7 +92,7 @@ tsk_fprintf(FILE * fd, const char *msg, ...)
     {
         WCHAR wbuf[2048];
         tsk_printf_conv(wbuf, 2048, msg, &args);
-        fwprintf(fd, _TSK_T("%s"), wbuf);
+        fwprintf(fd, _TSK_T("%ls"), wbuf);
     }
 #else
     vfprintf(fd, msg, args);
@@ -105,9 +105,9 @@ tsk_fprintf(FILE * fd, const char *msg, ...)
  * printf wrapper function that takes UTF-8 strings as input
  * (on all platforms) and does what is necessary to output
  * strings in the correct encoding (UTF-8 on Unix and
- * UTF-16 on Windows). 
+ * UTF-16 on Windows).
  *
- * @param msg printf message 
+ * @param msg printf message
  */
 void
 tsk_printf(const char *msg, ...)
@@ -119,10 +119,42 @@ tsk_printf(const char *msg, ...)
     {
         WCHAR wbuf[2048];
         tsk_printf_conv(wbuf, 2048, msg, &args);
-        wprintf(_TSK_T("%s"), wbuf);
+        wprintf(_TSK_T("%ls"), wbuf);
     }
 #else
     vprintf(msg, args);
 #endif
     va_end(args);
 }
+
+/**
+ * \ingroup baselib
+ * Prints the string removing control characters.
+ *
+ * @param fd File to print to
+ * @param str string to be printed
+ *
+ * @returns 0 on success and 1 on error
+ */
+int
+tsk_print_sanitized(FILE * fd, const char *str)
+{
+    size_t index = 0;
+    char *buf = NULL;
+
+    buf = tsk_malloc(strlen(str) + 1);
+    if (buf == NULL)
+      return 1;
+
+    strcpy(buf, str);
+
+    for (index = 0; index < strlen(buf); index++)
+      if (TSK_IS_CNTRL(buf[index]))
+        buf[index] = '^';
+
+    tsk_fprintf(fd, "%s", buf);
+
+    free(buf);
+
+    return 0;
+}
