@@ -1,12 +1,12 @@
 /*
 ** fatxxfs
-** The Sleuth Kit 
+** The Sleuth Kit
 **
-** Content and meta data layer support for the FATXX file system 
+** Content and meta data layer support for the FATXX file system
 **
 ** Brian Carrier [carrier <at> sleuthkit [dot] org]
 ** Copyright (c) 2006-2013 Brian Carrier, Basis Technology. All Rights reserved
-** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
+** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -20,15 +20,15 @@
 
 /**
  * \file fatxxfs.c
- * Contains the internal TSK FATXX (FAT12, FAT16, FAT32) file system code to 
- * handle basic file system processing for opening file system, processing 
- * sectors, and directory entries. 
+ * Contains the internal TSK FATXX (FAT12, FAT16, FAT32) file system code to
+ * handle basic file system processing for opening file system, processing
+ * sectors, and directory entries.
  */
 
 #include "tsk_fatxxfs.h"
 
 /*
- * Implementation NOTES 
+ * Implementation NOTES
  *
  * TSK_FS_META contains the first cluster.  file_walk will return sector
  * values though because the cluster numbers do not start until after
@@ -40,25 +40,24 @@
  * keep consistent with UNIX.  After that, each 32-byte slot is numbered
  * as though it were a directory entry (even if it is not).  Therefore,
  * when an inode walk is performed, not all inode values will be displayed
- * even when '-e' is given for ils. 
+ * even when '-e' is given for ils.
  *
  * Progs like 'ils -e' are very slow because we have to look at each
  * block to see if it is a file system structure.
  */
 
 /**
- * Print details about the file system to a file handle. 
+ * Print details about the file system to a file handle.
  *
  * @param fs File system to print details on
  * @param hFile File handle to print text to
- * 
+ *
  * @returns 1 on error and 0 on success
  */
 static uint8_t
 fatxxfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
 {
     unsigned int i;
-    int a;
     TSK_DADDR_T next, snext, sstart, send;
     FATFS_INFO *fatfs = (FATFS_INFO *) fs;
     FATXXFS_SB *sb = (FATXXFS_SB*)fatfs->boot_sector_buffer;
@@ -136,7 +135,7 @@ fatxxfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
             sb->a.f16.vol_lab[8], sb->a.f16.vol_lab[9],
             sb->a.f16.vol_lab[10]);
 
-        if ((vol_label_dentry) && (vol_label_dentry->name)) {
+        if ((vol_label_dentry) && (vol_label_dentry->name[0])) {
             tsk_fprintf(hFile,
                 "Volume Label (Root Directory): %c%c%c%c%c%c%c%c%c%c%c\n",
                 vol_label_dentry->name[0], vol_label_dentry->name[1], vol_label_dentry->name[2], vol_label_dentry->name[3],
@@ -156,7 +155,6 @@ fatxxfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
     else {
 
         char *fat_fsinfo_buf;
-        FATXXFS_FSINFO *fat_info;
 
         if ((fat_fsinfo_buf = (char *)
                 tsk_malloc(sizeof(FATXXFS_FSINFO))) == NULL) {
@@ -176,7 +174,7 @@ fatxxfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
             sb->a.f32.vol_lab[8], sb->a.f32.vol_lab[9],
             sb->a.f32.vol_lab[10]);
 
-        if ((vol_label_dentry) && (vol_label_dentry->name)) {
+        if ((vol_label_dentry) && (vol_label_dentry->name[0])) {
             tsk_fprintf(hFile,
                 "Volume Label (Root Directory): %c%c%c%c%c%c%c%c%c%c%c\n",
                 vol_label_dentry->name[0], vol_label_dentry->name[1], vol_label_dentry->name[2], vol_label_dentry->name[3],
@@ -196,9 +194,10 @@ fatxxfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
 
         /* Process the FS info */
         if (tsk_getu16(fs->endian, sb->a.f32.fsinfo)) {
+            FATXXFS_FSINFO *fat_info;
             cnt =
-                tsk_fs_read(fs, 
-                    (TSK_DADDR_T) tsk_getu16(fs->endian, sb->a.f32.fsinfo) * fs->block_size, 
+                tsk_fs_read(fs,
+                    (TSK_DADDR_T) tsk_getu16(fs->endian, sb->a.f32.fsinfo) * fs->block_size,
                     fat_fsinfo_buf, sizeof(FATXXFS_FSINFO));
 
             if (cnt != sizeof(FATXXFS_FSINFO)) {
@@ -352,12 +351,13 @@ fatxxfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
         fatfs->lastclust);
 
 
-    /* cycle via cluster and look at each cluster in the FAT 
+    /* cycle via cluster and look at each cluster in the FAT
      * for clusters marked as bad */
     cnt = 0;
     for (i = 2; i <= fatfs->lastclust; i++) {
         TSK_DADDR_T entry;
         TSK_DADDR_T sect;
+        unsigned int a;
 
         /* Get the FAT table entry */
         if (fatfs_getFAT(fatfs, i, &entry))
@@ -391,7 +391,7 @@ fatxxfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
     for (i = 2; i <= fatfs->lastclust; i++) {
 
         /* 'send' marks the end sector of the current run, which will extend
-         * when the current cluster continues to the next 
+         * when the current cluster continues to the next
          */
         send = FATFS_CLUST_2_SECT(fatfs, i + 1) - 1;
 
@@ -407,7 +407,7 @@ fatxxfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
         }
 
         /* The next clust is either further away or the clust is available,
-         * print it if is further away 
+         * print it if is further away
          */
         else if ((next & fatfs->mask)) {
             if (FATFS_ISEOF(next, fatfs->mask))
@@ -439,10 +439,8 @@ fatxxfs_open(FATFS_INFO *fatfs)
 	TSK_FS_INFO *fs = &(fatfs->fs_info);
 	FATXXFS_SB *fatsb = (FATXXFS_SB*)(&fatfs->boot_sector_buffer);
 	int i = 0;
-	ssize_t cnt = 0;
     TSK_DADDR_T sectors = 0;
 	TSK_FS_DIR * test_dir1; // Directories used to try opening the root directory
-	TSK_FS_DIR * test_dir2; //  to see if it's the Android FAT version
 
     // clean up any error messages that are lying around
     tsk_error_reset();
@@ -474,7 +472,7 @@ fatxxfs_open(FATFS_INFO *fatfs)
         return 1;
     }
 
-    // cluster size 
+    // cluster size
     fatfs->csize = fatsb->csize;
     if ((fatfs->csize != 0x01) &&
         (fatfs->csize != 0x02) &&
@@ -545,8 +543,8 @@ fatxxfs_open(FATFS_INFO *fatfs)
     }
 
     /* Calculate the block info
-     * 
-     * The sector of the begining of the data area  - which is 
+     *
+     * The sector of the beginning of the data area  - which is
      * after all of the FATs
      *
      * For TSK_FS_TYPE_FAT12 and TSK_FS_TYPE_FAT16, the data area starts with the root
@@ -558,7 +556,7 @@ fatxxfs_open(FATFS_INFO *fatfs)
         fatfs->sectperfat * fatfs->numfat;
 
     /* The sector where the first cluster is located.  It will be used
-     * to translate cluster addresses to sector addresses 
+     * to translate cluster addresses to sector addresses
      *
      * For TSK_FS_TYPE_FAT32, the first cluster is the start of the data area and
      * it is after the root directory for TSK_FS_TYPE_FAT12 and TSK_FS_TYPE_FAT16.  At this
@@ -631,7 +629,7 @@ fatxxfs_open(FATFS_INFO *fatfs)
 
     /* additional sanity checks if we think we are using the backup boot sector.
      * The scenario to prevent here is if fat_open is called 6 sectors before the real start
-     * of the file system, then we want to detect that it was not a backup that we saw.  
+     * of the file system, then we want to detect that it was not a backup that we saw.
      */
     if (fatfs->using_backup_boot_sector) {
         // only FAT32 has backup boot sectors..
@@ -650,6 +648,7 @@ fatxxfs_open(FATFS_INFO *fatfs)
             uint8_t buf2[512];
             int i2;
             int numDiffs;
+	        ssize_t cnt = 0;
 
             cnt =
                 tsk_fs_read(fs, fatfs->firstfatsect * fatfs->ssize,
@@ -758,7 +757,7 @@ fatxxfs_open(FATFS_INFO *fatfs)
     fs->root_inum = FATFS_ROOTINO;
     fs->first_inum = FATFS_FIRSTINO;
 
-    /* Calculate inode addresses for the virtual files (MBR, one or two FATS) 
+    /* Calculate inode addresses for the virtual files (MBR, one or two FATS)
      * and the virtual orphan files directory. */
     fs->last_inum = (FATFS_SECT_2_INODE(fatfs, fs->last_block_act + 1) - 1) + FATFS_NUM_VIRT_FILES(fatfs);
     fatfs->mbr_virt_inum = fs->last_inum - FATFS_NUM_VIRT_FILES(fatfs) + 1;
@@ -784,7 +783,7 @@ fatxxfs_open(FATFS_INFO *fatfs)
     }
 
     /*
-     * Set the function pointers  
+     * Set the function pointers
      */
 
     fs->block_walk = fatfs_block_walk;
@@ -830,11 +829,13 @@ fatxxfs_open(FATFS_INFO *fatfs)
 	fatfs->subtype = TSK_FATFS_SUBTYPE_SPEC;
 	test_dir1 = tsk_fs_dir_open_meta(fs, fs->root_inum);
 
-	if(test_dir1->names_used <= 4){ // At most four automatic directores ($MBR, $FAT1, $FAT1, $OrphanFiles)
+	if (test_dir1 != NULL && test_dir1->names_used <= 4){ // At most four automatic directories ($MBR, $FAT1, $FAT1, $OrphanFiles)
+	    TSK_FS_DIR * test_dir2; //  to see if it's the Android FAT version
+
 		fatfs->subtype = TSK_FATFS_SUBTYPE_ANDROID_1;
 		test_dir2 = tsk_fs_dir_open_meta(fs, fs->root_inum);
 
-		if(test_dir2->names_used > test_dir1->names_used){
+		if (test_dir2 != NULL && test_dir2->names_used > test_dir1->names_used){
 			fatfs->subtype = TSK_FATFS_SUBTYPE_ANDROID_1;
 		}
 		else{
