@@ -23,7 +23,7 @@
  * Contains the short (1 character) name of the file type
  */
 char tsk_fs_meta_type_str[TSK_FS_META_TYPE_STR_MAX][2] =
-    { "-", "r", "d", "p", "c", "b", "l", "s", "h", "w", "v"
+    { "-", "r", "d", "p", "c", "b", "l", "s", "h", "w", "v", "V"
 };
 
 /**
@@ -50,27 +50,32 @@ tsk_fs_meta_alloc(size_t a_buf_len)
             return NULL;
         }
         fs_meta->content_len = a_buf_len;
+        fs_meta->reset_content = NULL;
     }
 
     // assign the id so we know the structure is still alloc
     fs_meta->tag = TSK_FS_META_TAG;
 
-    return (fs_meta);
+    return fs_meta;
 }
 
 
 /**
  * \internal
  * Resize an existing TSK_FS_META structure -- changes the number of
- * block pointers. 
+ * block pointers.
  *
  * @param fs_meta Structure to resize
  * @param a_buf_len Size of file system specific data that is used to store references to file content
- * @return NULL on error 
+ * @return NULL on error
  */
 TSK_FS_META *
 tsk_fs_meta_realloc(TSK_FS_META * a_fs_meta, size_t a_buf_len)
 {
+    if (a_fs_meta->reset_content != NULL) {
+        a_fs_meta->reset_content(a_fs_meta->content_ptr);
+    }
+
     if (a_fs_meta->content_len != a_buf_len) {
         a_fs_meta->content_len = a_buf_len;
         a_fs_meta->content_ptr =
@@ -79,7 +84,7 @@ tsk_fs_meta_realloc(TSK_FS_META * a_fs_meta, size_t a_buf_len)
             return NULL;
         }
     }
-    return (a_fs_meta);
+    return a_fs_meta;
 }
 
 
@@ -100,8 +105,14 @@ tsk_fs_meta_close(TSK_FS_META * fs_meta)
     // clear the tag so we know the structure isn't alloc
     fs_meta->tag = 0;
 
-    if (fs_meta->content_ptr)
+    if (fs_meta->content_ptr) {
+        if (fs_meta->reset_content) {
+            fs_meta->reset_content(fs_meta->content_ptr);
+        }
+
         free((char *) fs_meta->content_ptr);
+    }
+
     fs_meta->content_ptr = NULL;
     fs_meta->content_len = 0;
 
@@ -109,8 +120,7 @@ tsk_fs_meta_close(TSK_FS_META * fs_meta)
         tsk_fs_attrlist_free(fs_meta->attr);
     fs_meta->attr = NULL;
 
-    if (fs_meta->link)
-        free(fs_meta->link);
+    free(fs_meta->link);
     fs_meta->link = NULL;
 
     fs_name = fs_meta->name2;
@@ -121,7 +131,7 @@ tsk_fs_meta_close(TSK_FS_META * fs_meta)
         fs_name = fs_name2;
     }
 
-    free((char *) fs_meta);
+    free(fs_meta);
 }
 
 /** \internal
@@ -137,6 +147,11 @@ tsk_fs_meta_reset(TSK_FS_META * a_fs_meta)
     TSK_FS_META_NAME_LIST *name2_tmp;
     char *link_tmp;
 
+    // Clear content
+    if (a_fs_meta->reset_content) {
+        a_fs_meta->reset_content(a_fs_meta->content_ptr);
+    }
+
     // backup pointers
     content_ptr_tmp = a_fs_meta->content_ptr;
     content_len_tmp = a_fs_meta->content_len;
@@ -178,7 +193,7 @@ tsk_fs_meta_reset(TSK_FS_META * a_fs_meta)
  * \ingroup fslib
  * Walk a range of metadata structures and call a callback for each
  * structure that matches the flags supplied.   For example, it can
- * call the callback on only allocated or unallocated entries. 
+ * call the callback on only allocated or unallocated entries.
  *
  * @param a_fs File system to process
  * @param a_start Metadata address to start walking from
